#include <vdr/recording.h>

#include "tools/socket.h"
#include "tools/select.h"

#include "server/recordingstreamer.h"
#include "recplayer.h"
#include "common.h"

// --- cStreamdevRecStreamer -------------------------------------------------

cStreamdevRecStreamer::cStreamdevRecStreamer(const cRecording *Recording):
		cThread("streamdev-recordingstreaming"),
		m_Recording(Recording),
		m_RecPlayer(new cRecPlayer(Recording)),
		m_Position(0),
		m_TotalBytesWritten(0)
{
}

cStreamdevRecStreamer::~cStreamdevRecStreamer()
{
	Dprintf("Desctructing Recording streamer\n");
	Stop();

	DELETENULL(m_RecPlayer);
}

void cStreamdevRecStreamer::Start(cTBSocket *Socket)
{
	Dprintf("start streamer\n");
	m_Socket = Socket;
	cThread::Start();
}

void cStreamdevRecStreamer::Stop(void)
{
	if (Running()) {
		Dprintf("stopping streamer\n");
		Cancel(3);
	}
}

uint64_t cStreamdevRecStreamer::getLengthBytes()
{
	return m_RecPlayer->getLengthBytes();
}

uint32_t cStreamdevRecStreamer::getLengthFrames()
{
	return m_RecPlayer->getLengthFrames();
}

void cStreamdevRecStreamer::seekPosition(uint64_t position)
{
	m_Position = position;
}

void cStreamdevRecStreamer::Action(void)
{
	cTBSelect sel;
	uint8_t data[32768];
	Dprintf("Writer start\n");

	sel.Clear();
	sel.Add(*m_Socket, true);
	while (Running()) {

		if (sel.Select(15000) == -1) {
			esyslog("ERROR: streamdev-server: couldn't send recording data: %m");
			continue; /* Continue here instead of break, the recording playback can be paused */
		}

		if (sel.CanWrite(*m_Socket)) {
			int written;
			unsigned long readed = m_RecPlayer->getBlock(&*data, m_Position, sizeof(data));
			if (readed <= 0)
				continue;

			if ((written = m_Socket->Write(&*data, readed)) == -1) {
				esyslog("ERROR: streamdev-server: couldn't send %d bytes: %m", written);
				break;
			}

			m_Position += written;
			m_TotalBytesWritten += written;
		}
	}
}
